/*
 * test.js
 *
 * USAGE: cscript test.js basePath testProgFile testProgFilename 
 * profileConfig scriptingVisible logFilePath profileResultsPath 
 * breakpointInFile breakpointLineInFile atkSrcPath ccsInstallDir simFilesPath
 * mcbspNumber dxrDrrAddress
 */ 
testEnv = {
    // base path to .out, script, profile content etc
    basePath: WScript.Arguments(0),
	
	  // Set test program (location of .out relative to basePath)
    testProgFile: WScript.Arguments(1),

    // Set test program filename, without the extension 
    testProgFilename: WScript.Arguments(2),
       
    // Which profile configuration (.ini)
    profileConfig: WScript.Arguments(3),
        
    // do we wish to see CCS GUI during script (for test) or run in background?
    scriptingVisible: WScript.Arguments(4),
    
    // Paths to log file and profile results 
    logFilePath: WScript.Arguments(5),
    profileResultsPath: WScript.Arguments(6), 
	
    // breakpoint attributes required (fxn name)
    breakpointFxnName: WScript.Arguments(7), 
    
	  // ; delimited paths to source code for ATK 
    atkSrcPath: WScript.Arguments(8),
        
    // CCS Install path (used to locate ATK executables)
    ccsInstallDir: WScript.Arguments(9),

    // Sim files path (for port/pin connect files)  
    simFilesPath: WScript.Arguments(10),
   
    // Which MCBSP to use? (eg dsk6416 uses MCBSP2, dsk6713 uses MCBSP1)
    mcbspNumber: WScript.Arguments(11),
    
    // Address of the DXR & DRR for the given MCBSP
    dxrDrrAddress: WScript.Arguments(12),

    boardName: "",
    cpuName: "",
    majorISA: "",
    minorISA: ""
};

/* generic return code variable */
var ret_val;

var WshShell = WScript.CreateObject("WScript.Shell");
var ccs = WScript.CreateObject("CCS_Scripting_Com.CCS_Scripting"); 

// Open log file
ccs.ScriptTraceBegin(testEnv.basePath + testEnv.logFilePath + "\\log_file.txt");    
ccs.ScriptTraceVerbose(VERBOSE_ALL);

// open whichever config is in CCS Setup right now
ccs.CCSOpenNamed("*", "*", testEnv.scriptingVisible); 

testEnv.boardName = ccs.TargetGetBoardName();
testEnv.cpuName = ccs.TargetGetCPUName();
testEnv.majorISA = ccs.TargetGetMajorISA();
testEnv.minorISA = ccs.TargetGetMinorISA();
ccs.ScriptTraceWrite("TEST: opened CCS connection: " + testEnv.boardName + " - " + 
    testEnv.cpuName + " - " + testEnv.majorISA + " - " + testEnv.minorISA + "\n"); 

/* Output Command line arguments as sanity check */
PrintWScriptArgs();

// Construct .tprof filename 
testEnv.testProgFile = testEnv.basePath + testEnv.testProgFile;
var splitPath = testEnv.testProgFile.split(/[/\\]/);  // Split into chunks delimited by / or \
var lengthOfFilename = splitPath[splitPath.length - 1].length;  // Length of filename including extension
var tprofFilename = testEnv.testProgFile.substring(0, testEnv.testProgFile.length - lengthOfFilename - 1);
tprofFilename = tprofFilename + "\\" + testEnv.testProgFilename + ".tprof";

/*
 *  Profiling of RF3 Sim64xx is done as follows (>= CCS 3.1) :-
 *  - cpu reset
 *  - load .out
 *  - connect ports/pins for McBSP data hookup via gel 'Connect'
 *  - set a BP in 1 of the main frame-based fxns eg thrRxSplitRun 1st line
 *  - run to this BP
 *  - run to this BP again (now the cache is in steady-state proxy)
 *  - load profile configuration : to pick up 'Collect code coverage data' attribute
 *  - Enable Profiling
 *  - run to the BP again
 *  - Disable Profiling
 *  - run ATK executables to generate the cycles data
 *  - Clear breakpoints 
 *  - exit
 */

// reset CPU so that numbers are truly same on each run
ccs.TargetReset();
ccs.ScriptTraceWrite("TEST: reset\n");

// load the .out that we passed in via cmd line
ccs.ProgramLoad(testEnv.testProgFile); 
ccs.ScriptTraceWrite("TEST: loaded\n"); 

// Remove any stale tprof file
ret_val = WshShell.run("cmd /c del \"" + tprofFilename + "\"", 0, true); 

// flip '\' to '/' in the basePath for passing paths to GEL expressions. 
//   Because the usage of '\' requires a preceding escape character, we prefer
//   to bail and use '/' instead.
var basePathFwdSlash = testEnv.basePath.replace(/\\/g, "/");
testEnv.simFilesPath = testEnv.simFilesPath.replace(/\\/g, "/");

// do port connect for McBSP data  
var tmp;
ccs.ScriptTraceWrite("TEST: Connecting Ports...\n"); 
tmp = basePathFwdSlash + testEnv.simFilesPath + "/input_data.txt";
ccs.TargetEvalExpression('GEL_PortConnect(' + testEnv.dxrDrrAddress + ', 1, 4, 1, "' + tmp + '" )' );  
tmp = basePathFwdSlash + testEnv.simFilesPath + "/output_data.txt";
ccs.TargetEvalExpression('GEL_PortConnect(' + testEnv.dxrDrrAddress + ', 1, 4, 2, "' + tmp + '" )' );  

// do pin connect for McBSP data  
var pin;
tmp = basePathFwdSlash + testEnv.simFilesPath + "/CLKX" + testEnv.mcbspNumber + ".txt";
pin = "CLKX" + testEnv.mcbspNumber;
ccs.ScriptTraceWrite("TEST: Connecting Pins...\n"); 
ccs.TargetEvalExpression('GEL_PinConnect("' + pin + '", "' + tmp + '" )' );  
tmp = basePathFwdSlash + testEnv.simFilesPath + "/CLKR" + testEnv.mcbspNumber + ".txt";
pin = "CLKR" + testEnv.mcbspNumber;
ccs.TargetEvalExpression('GEL_PinConnect("' + pin + '", "' + tmp + '" )' );  
tmp = basePathFwdSlash + testEnv.simFilesPath + "/FSR" + testEnv.mcbspNumber + ".txt";
pin = "FSR" + testEnv.mcbspNumber;
ccs.TargetEvalExpression('GEL_PinConnect("' + pin + '", "' + tmp + '" )' );  
tmp = basePathFwdSlash + testEnv.simFilesPath + "/FSX" + testEnv.mcbspNumber + ".txt";
pin = "FSX" + testEnv.mcbspNumber;
ccs.TargetEvalExpression('GEL_PinConnect("' + pin + '", "' + tmp + '" )' );  
ccs.TargetEvalExpression('GEL_TextOut("Ports and pins successfully connected.\n");'); 

// set breakpoint at fxn we wish to begin profiling session from
var bpFxnAddress = ccs.SymbolGetAddress(testEnv.breakpointFxnName);
ccs.BreakpointSetAddress(bpFxnAddress);

// run to 1st breakpoint
ccs.TargetRun(); 
ccs.ScriptTraceWrite("TEST: ran to breakpoint\n");

// run to same breakpoint again to put the cache in steady state
ccs.TargetRun(); 
ccs.ScriptTraceWrite("TEST: ran to 2nd breakpoint\n");

// load the profile config ie Code Coverage & Exclusive counts selected
testEnv.profileConfig = testEnv.basePath + testEnv.profileConfig;
ccs.LoadProfileConfiguration(testEnv.profileConfig);

// enable profiling for this frame
ccs.EnableProfiling();

// run to same breakpoint again now collecting profiling data
ccs.TargetRun(); 
ccs.ScriptTraceWrite("TEST: ran to 3rd breakpoint\n");

// need to disable profiling in order to run Code Coverage viewer 
ccs.DisableProfiling();

// remove any stale files from previous runs
WshShell.run("cmd /c del \"*.csv\"", 0, true);
testEnv.profileResultsPath = testEnv.basePath + testEnv.profileResultsPath;
WshShell.run("cmd /c del \"" + testEnv.profileResultsPath + "\\"
      + testEnv.testProgFilename + "_summary.xls\"", 0, true);

// Generate ATK results
ret_val = ATKtprof2xls(testEnv.ccsInstallDir,
             testEnv.testProgFile,
             testEnv.testProgFilename,
             tprofFilename,
             testEnv.profileResultsPath + "\\" + testEnv.testProgFilename 
                  + "_summary.xls",
             testEnv.atkSrcPath);

ccs.BreakpointRemoveAll();  // Clean up all breakpoints
ccs.ScriptTraceWrite("TEST: ran ATK coverage & cov2xls. ret_val = " + ret_val 
      + "\n");
             
ccs.ScriptTraceWrite("\nTEST: The End\n");
ccs.ScriptTraceEnd();    // Close log file
ccs.CCSClose(1);

delete testEnv; 

